# IRFsubsets.R
#
# Generates plots of subsets of the IRFs for the three models
#
# Patrick Brandt
#
# April 3, 2006    -- original version
# May 17, 2006     -- Revised to generate some new responses and changes
#                     the resolution of the output figures.
# May 25, 2006     -- Increased size of the IR shock to real econ graph.
# December 4, 2006 -- Changed the sign normalization to be that
#                     generated by the models as estimated.

library(MSBVAR)

# New function that uses a list of MCMC generated impulse responses
# and knows how to combine them into one set of graphs with smart
# coloring / line types.

plot.mc.irf.svar.subset <- function(mc.impulse.list, num.eqn, num.shk,
                                    probs = c(0.16, 0.84),
                                    eqnnames = NULL,
                                    shknames = NULL, ...)

{
    total <- num.eqn*num.shk
    m <- num.eqn
    nsteps <- dim(mc.impulse.list[[1]])[2]
    draws <- dim(mc.impulse.list[[1]])[1]

    # Now build a set of lists of arrays to store the irf summaries
    irf.storage <- array(0, c(nsteps, length(probs) + 1, total,
                              length(mc.impulse.list)))

    irf.ci <- array(0, c(nsteps, length(probs) + 1, total))

    for (j in 1:length(mc.impulse.list))
    {
        for (i in 1:total) {
            irf.bands <- t(apply(mc.impulse.list[[j]][, , i], 2, quantile,
                                 probs))
            irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, median)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
        irf.storage[,,,j] <- irf.ci
    }

    # Clean up
    rm(irf.ci)
    gc()

    # Now find the min-max values for the plots
    minmax <- array(0, c(num.eqn, 2, length(mc.impulse.list)))


    if(num.eqn>1)
    {
        for (j in 1:length(mc.impulse.list))
        {
            within.plots <- apply(irf.storage[,,,j], 3, range)
            tmp <- (c(1:total)%%num.eqn)
            tmp[tmp == 0] <- num.eqn
            indices <- sort(tmp, index.return = T)$ix
            dim(indices) <- c(num.shk, num.eqn)
            for (i in 1:num.eqn) {
                minmax[i,,j] <- range(within.plots[, indices[, i]])
            }
        }
    }else{ minmax <- array(range(irf.storage), c(num.eqn, 2,
    length(mc.impulse.list))) }

    # Now do the plotting
    j <- 1
    par(mfcol = c(num.eqn, num.shk), mai = c(0.15, 0.2, 0.25, 0.15), omi = c(0.25,
        0.85, 1, 0.15))
    for (i in 1:total) {
        lims <- ifelse((i-num.eqn)%%num.eqn==0, num.eqn,
                       (i-num.eqn)%%num.eqn)

        tmp <- matrix(irf.storage[, , i,], nrow=nsteps,
                      ncol=3*dim(irf.storage)[4])

        ts.plot(tmp,
                gpars = list(xlab = "",
                lty=rep(seq(1:length(mc.impulse.list)), 1, each=3),
                ylab = "", #col=c(rep("black",3), rep("red", 3)),
                lwd=1,
                ylim = 1.1*range(minmax[lims,,])))
        abline(h = 0)
        if(i<=num.eqn)
        { mtext(eqnnames[i], side=2, line=3) }
        if((i-1)%%num.eqn==0)
        { mtext(shknames[j], side=3, line=2)
          j <- j+1
      }

    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
}

# Load the IRFs and get the responses we want to see

politics2policy.subset <- c(79,90,101,112)
policy2politics.subset <- c(19:22)
XR2politics.subset <- c(8:11)
IR2XR.subset <- 12
econ2politics.subset <- c(41:44,52:55)
IR2Real.subset <- c(15,16)
PoliticalBlock.subset <- c(85:88, 96:99, 107:110, 118:121)

# Load and subset Model 1: No accountability responses
load("Model1.RData")
rm(Model1.A0.posterior.obj)
gc()

NoAcct.Politics2Policy.IRF <- 100*Model1.impulses[,, politics2policy.subset]
NoAcct.Policy2Politics.IRF <- 100*Model1.impulses[,, policy2politics.subset]
NoAcct.XR2Politics.IRF <- 100*Model1.impulses[,, XR2politics.subset]
NoAcct.IR2XR.IRF <- 100*Model1.impulses[,, IR2XR.subset]
dim(NoAcct.IR2XR.IRF) <- c(N2,12,1)
NoAcct.Econ2Politics.IRF <- 100*Model1.impulses[,, econ2politics.subset]
NoAcct.IR2Real.IRF <- 100*Model1.impulses[,, IR2Real.subset]
NoAcct.IR2Real.IRF[,,1] <-  1*NoAcct.IR2Real.IRF[,,1]
NoAcct.PoliticalBlock.IRF <- 100*Model1.impulses[,, PoliticalBlock.subset]
rm(Model1.impulses)
gc()


# Load and subset Model 2: Reaction only responses
load("Model2.RData")
rm(Model2.A0.posterior.obj)
gc()

Reaction.Politics2Policy.IRF <- 100*Model2.impulses[,, politics2policy.subset]
Reaction.Policy2Politics.IRF <- 100*Model2.impulses[,, policy2politics.subset]
Reaction.XR2Politics.IRF <- 100*Model2.impulses[,, XR2politics.subset]
Reaction.IR2XR.IRF <- 100*Model2.impulses[,, IR2XR.subset]
dim(Reaction.IR2XR.IRF) <- c(N2,12,1)
Reaction.Econ2Politics.IRF <- 100*Model2.impulses[,, econ2politics.subset]
Reaction.Econ2Politics.IRF[,,1:3] <- 100*Reaction.Econ2Politics.IRF[,,1:3]
Reaction.IR2Real.IRF <- 100*Model2.impulses[,, IR2Real.subset]
Reaction.PoliticalBlock.IRF <- 100*Model2.impulses[,, PoliticalBlock.subset]

rm(Model2.impulses)
gc()


# Load and subset Model 3: Accountability responses
load("Model3.RData")
rm(Model3.A0.posterior.obj)
gc()

Acct.Politics2Policy.IRF <- 100*Model3.impulses[,, politics2policy.subset]
Acct.Policy2Politics.IRF <- 100*Model3.impulses[,, policy2politics.subset]
Acct.XR2Politics.IRF <- 100*Model3.impulses[,, XR2politics.subset]
Acct.IR2XR.IRF <- 100*Model3.impulses[,, IR2XR.subset]
dim(Acct.IR2XR.IRF) <- c(N2,12,1)
Acct.Econ2Politics.IRF <- 100*Model3.impulses[,, econ2politics.subset]
Acct.Econ2Politics.IRF[,,c(1,4)] <- 100*Acct.Econ2Politics.IRF[,,c(1,4)]
Acct.IR2Real.IRF <- 100*Model3.impulses[,, IR2Real.subset]
Acct.PoliticalBlock.IRF <- 100*Model3.impulses[,, PoliticalBlock.subset]

rm(Model3.impulses)
gc()


# Now plot the various responses

postscript(file="PoliticsShk2Policy-noflips.eps", width=6, height=3,
           bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.Politics2Policy.IRF,
                             Reaction.Politics2Policy.IRF,
                             Acct.Politics2Policy.IRF),
                        num.eqn=1, num.shk=4,
                        probs = c(0.16, 0.84),
                        eqnnames = "IR",
                        shknames = c("SE","PE","PM","VI"))
dev.off()

postscript(file="PolicyShk2Politics-noflips.eps", width=2.5, height = 6, bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.Policy2Politics.IRF,
                             Reaction.Policy2Politics.IRF,
                             Acct.Policy2Politics.IRF),
                        num.eqn=4, num.shk=1,
                        probs = c(0.16, 0.84),
                        eqnnames = c("SE","PE","PM","VI"),
                        shknames = "IR")
dev.off()

postscript(file="XRShk2Politics-noflips.eps", width=2.5, height=6, bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.XR2Politics.IRF,
                             Reaction.XR2Politics.IRF,
                             Acct.XR2Politics.IRF),
                        num.eqn=4, num.shk=1,
                        probs = c(0.16, 0.84),
                        eqnnames = c("SE","PE","PM","VI"),
                        shknames = "XR")
dev.off()

postscript(file="IRShk2XR-noflips.eps", width=3.5, height=4, bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.IR2XR.IRF,
                             Reaction.IR2XR.IRF,
                             Acct.IR2XR.IRF),
                        num.eqn=1, num.shk=1,
                        probs = c(0.16, 0.84),
                        eqnnames = "XR",
                        shknames = "IR")

dev.off()

postscript(file="EconShk2Politics-noflips.eps", width=5, height=6, bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.Econ2Politics.IRF,
                             Reaction.Econ2Politics.IRF,
                             Acct.Econ2Politics.IRF),
                        num.eqn=4, num.shk=2,
                        probs = c(0.16, 0.84),
                        eqnnames = c("SE","PE","PM","VI"),
                        shknames = c("CPI","IIP"))
dev.off()

## pdf(file="EconShk2Politics.pdf", width=6, height=8
##     )
## plot.mc.irf.svar.subset(list(NoAcct.Econ2Politics.IRF,
##                              Reaction.Econ2Politics.IRF,
##                              Acct.Econ2Politics.IRF),
##                         num.eqn=4, num.shk=2,
##                         probs = c(0.16, 0.84),
##                         eqnnames = c("SE","PE","PM","VI"),
##                         shknames = c("CPI","IIP"))
## dev.off()

postscript(file="IRShk2RealEcon-noflips.eps", width=3.5, height=6, bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.IR2Real.IRF,
                             Reaction.IR2Real.IRF,
                             Acct.IR2Real.IRF),
                        num.eqn=2, num.shk=1,
                        probs = c(0.16, 0.84),
                        shknames = c("IR"),
                        eqnnames = c("CPI","IIP"))
dev.off()


postscript(file="PoliticalBlock-noflips.eps", width=6, height=6, bg="transparent",
           horizontal = TRUE, onefile = TRUE, paper = "special")
plot.mc.irf.svar.subset(list(NoAcct.PoliticalBlock.IRF,
                             Reaction.PoliticalBlock.IRF,
                             Acct.PoliticalBlock.IRF),
                        num.eqn=4, num.shk=4,
                        probs = c(0.16, 0.84),
                        eqnnames = c("SE","PE","PM","VI"),
                        shknames = c("SE","PE","PM","VI"))
dev.off()

q(save="no")
